/*
 * Copyright (c) 2022 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */


#include <zephyr/logging/log.h>
#include <nrf.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/kernel.h>
#include <zephyr/types.h>
#include <zephyr/sys/printk.h>
#include <zephyr/ipc/ipc_service.h>

#define STACKSIZE	(1024)
K_THREAD_STACK_DEFINE(ipc0_stack, STACKSIZE);

LOG_MODULE_REGISTER(host, LOG_LEVEL_INF);

static const struct device *ipc0_instance = DEVICE_DT_GET(DT_NODELABEL(ipc0));
static struct ipc_ept ep;

static K_SEM_DEFINE(bound_sem, 0, 1);


static void ep_bound(void *priv)
{
	//LOG_INF("EP BOUND");
	k_sem_give(&bound_sem);
}
static void ep_recv(void *priv)
{
	printk("Msg received");
    
}

static struct ipc_ept_cfg ep_cfg = {
	.name = "ep0",
	.cb = {
		.bound    = ep_bound,
		.received = ep_recv,
	},
};


/** @brief Allow access to specific GPIOs for the network core.
 *
 * Function is executed very early during system initialization to make sure
 * that the network core is not started yet. More pins can be added if the
 * network core needs them.
 */
static int network_gpio_allow(const struct device *dev)
{
	ARG_UNUSED(dev);

	/* When the use of the low frequency crystal oscillator (LFXO) is
	 * enabled, do not modify the configuration of the pins P0.00 (XL1)
	 * and P0.01 (XL2), as they need to stay configured with the value
	 * Peripheral.
	 */
	NRF_P0_S->PIN_CNF[28] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);

	return 0;
}

SYS_INIT(network_gpio_allow, PRE_KERNEL_1, CONFIG_KERNEL_INIT_PRIORITY_OBJECTS);

void main(void)
{
	printk("Hello world from CPUAPP %s\n", CONFIG_BOARD);
	
	int ret;
	
	ret = ipc_service_open_instance(ipc0_instance);
	if ((ret < 0) && (ret != -EALREADY)) {
		LOG_ERR("ipc_service_open_instance() failure - ret: %d", ret);
		return;
	}
	LOG_INF("IPC service opened complete - ret: %d", ret);

	ret = ipc_service_register_endpoint(ipc0_instance, &ep, &ep_cfg);
	if (ret < 0) {
		LOG_ERR("ipc_service_register_endpoint() failure");
		return;
	}
	LOG_INF("EP Registered complete");

	k_sem_take(&bound_sem, K_FOREVER);

	while(true){

		__WFE();
	}

}
